
package de.codedo.conflicteditor;

import java.io.IOException;
import java.io.InputStream;
import java.net.MalformedURLException;
import java.net.URL;

import org.codehaus.jackson.JsonNode;
import org.codehaus.jackson.map.ObjectMapper;

/**
 * Creates a temporary view in the database to retrieve all documents that have conflicts.
 */
public class ConflictsView extends Object
{
    // @formatter:off
    private static final String VIEW =
        "function(doc) {" +
        "    if(doc._conflicts) {" +
        "        emit(doc, null);" +
        "    }" +
        "}";
    // @formatter:on

    public static final String JSON = String.format("{ \"map\": \"%s\" }", VIEW);

    private static final String TEMP_VIEW_URL_PATH = "_temp_view";

    private HttpAccess _httpAccess = null;

    private URL _viewUrl;

    public ConflictsView(CouchDb database) throws MalformedURLException
    {
        super();
        _viewUrl = database.urlByAddingPath(TEMP_VIEW_URL_PATH);
    }

    public ConflictsView(HttpAccess httpAccess)
    {
        super();
        _httpAccess = httpAccess;
    }

    public JsonNode getConflicts() throws IOException
    {
        InputStream input = null;
        try
        {
            input = getHttpAccess().post(JSON);
            return rowsNodeFromJson(input);
        }
        finally
        {
            if (input != null)
            {
                input.close();
            }
        }
    }

    private HttpAccess getHttpAccess()
    {
        if (_httpAccess == null)
        {
            _httpAccess = new UrlConnectionHttpAccess(_viewUrl);
        }
        return _httpAccess;
    }

    private JsonNode rowsNodeFromJson(InputStream input) throws IOException
    {
        ObjectMapper mapper = new ObjectMapper();
        JsonNode rootNode = mapper.readTree(input);
        return rootNode.get("rows");
    }
}
